-- A few globals
--
SIDEBAR_DEBUG=1;
Left_Sidebar_On = 1;
Right_Sidebar_On = 1;
SideBar_HideKeys = 0;
SideBar_HideKeyMod = 0;

SideBar_LeftOffset = 0;
SideBar_RightOffset = 0;

SideBar_NumberOfButtons = 6;
SideBar2_NumberOfButtons = 6;

SideBar_TotalNumberOfButtons = 12;
SideBar2_TotalNumberOfButtons = 12;

SideBarButton_IDOffset = 9*12;
SideBar2Button_IDOffset = 9.5*12;

-- handling sidebar button IDs

function SideBar_GetMaximumID()
	return (NUM_ACTIONBAR_PAGES * NUM_ACTIONBAR_PAGES);
end

function SideBar_RestrainOffset(offset, numberOfButtons)
	local maximum = SideBar_GetMaximumID() - numberOfButtons;
	if ( offset < 1 ) then
		offset = 1;
	elseif(offset > maximum ) then
		offset = maximum;
	end
	return offset;
end

function SideBar_SetIDOffset(offset)
	SideBarButton_IDOffset = SideBar_RestrainOffset(offset, SideBar_NumberOfButtons);
end

function SideBar2_SetIDOffset(offset)
	SideBar2Button_IDOffset = SideBar_RestrainOffset(offset, SideBar2_NumberOfButtons);
end

function SideBar_GetIDOffset(bar, secondHalf)
	local offset = (bar-1) * NUM_ACTIONBAR_PAGES;
	if ( secondHalf ) then
		offset = offset + ceil( NUM_ACTIONBAR_PAGES / 2);
	end
	return offset;
end

function SideBar_SetActionBar(bar, secondHalf)
	SideBar_SetIDOffset(SideBar_GetIDOffset(bar, secondHalf));
end

function SideBar2_SetActionBar(bar, secondHalf)
	SideBar2_SetIDOffset(SideBar_GetIDOffset(bar, secondHalf));
end

function SideBar_RestrainNumberOfButtons(numberOfButtons, totalNumberOfButtons)
	if ( numberOfButtons < 0 ) then
		numberOfButtons = 0;
	elseif ( numberOfButtons > totalNumberOfButtons ) then
		numberOfButtons = totalNumberOfButtons;
	end
	return numberOfButtons;
end

function SideBar_SetNumberOfButtons(numberOfButtons)
	SideBar_NumberOfButtons = SideBar_RestrainNumberOfButtons(numberOfButtons, SideBar_TotalNumberOfButtons);
end

function SideBar2_SetNumberOfButtons(numberOfButtons)
	SideBar2_NumberOfButtons = SideBar_RestrainNumberOfButtons(numberOfButtons, SideBar2_TotalNumberOfButtons);
end

-- main functions

function SideBarButtonDown(id)
	local button = getglobal("SideBarButton"..id);
	DebugPrint(button, "SIDEBAR_DEBUG");
	if ( button:GetButtonState() == "NORMAL" ) then
		button:SetButtonState("PUSHED");
	end
end

function SideBarButtonUp(id)
	local button = getglobal("SideBarButton"..id);
	if ( button:GetButtonState() == "PUSHED" ) then
		button:SetButtonState("NORMAL");
		-- Used to save a macro
		MacroFrame_EditMacro();
		UseAction(SideBarButton_GetID(button), 0);
		if ( IsCurrentAction(SideBarButton_GetID(button)) ) then
			button:SetChecked(1);
		else
			button:SetChecked(0);
		end
	end
end

function SideBar2ButtonDown(id)
	local button = getglobal("SideBar2Button"..id);
	DebugPrint(button, "SIDEBAR_DEBUG");
	if ( button:GetButtonState() == "NORMAL" ) then
		button:SetButtonState("PUSHED");
	end
end

function SideBar2ButtonUp(id)
	local button = getglobal("SideBar2Button"..id);
	if ( button:GetButtonState() == "PUSHED" ) then
		button:SetButtonState("NORMAL");
		-- Used to save a macro
		MacroFrame_EditMacro();
		UseAction(SideBarButton_GetID(button), 0);
		if ( IsCurrentAction(SideBarButton_GetID(button)) ) then
			button:SetChecked(1);
		else
			button:SetChecked(0);
		end
	end
end

function SideBar_UpdateOffset()
	SideBar2:SetPoint("LEFT", "UIParent", "LEFT", 0, SideBar_LeftOffset);
	SideBar:SetPoint("RIGHT", "UIParent", "RIGHT", 0, SideBar_RightOffset);
end

function SideBar_LeftSliderUpdate(toogle, speed)
	SideBar_LeftOffset = speed;
	SideBar_UpdateOffset();
end

function SideBar_RightSliderUpdate(toogle, speed)
	SideBar_RightOffset = speed;
	SideBar_UpdateOffset();
end

function SideBar_ID_LeftSliderUpdate(toogle, value)
	SideBar2_SetActionBar(value);
	SideBar2_Update();
end

function SideBar_ID_RightSliderUpdate(toogle, value)
	SideBar_SetActionBar(value);
	SideBar_Update();
end

function SideBar_LeftSliderNumberButtonsUpdate(toogle, value)
	SideBar2_SetNumberOfButtons(value);
	SideBar2_Update();
end

function SideBar_RightSliderNumberButtonsUpdate(toogle, value)
	SideBar_SetNumberOfButtons(value);
	SideBar_Update();
end

function SideBar_OnLoad()
	if ( Cosmos_RegisterConfiguration ) then
		Cosmos_RegisterConfiguration("COS_SIDEBAR", "SECTION", SIDEBAR_SEP, SIDEBAR_SEP_INFO );
		Cosmos_RegisterConfiguration("COS_SIDEBARHEADER", "SEPARATOR", SIDEBAR_SEP, SIDEBAR_SEP_INFO );
		Cosmos_RegisterConfiguration("COS_SIDEBARL", "CHECKBOX",
			SIDEBAR_CHECK1, 
			SIDEBAR_CHECK1_INFO,
			SideBar2_Toggle,
			Left_Sidebar_On
			);
		Cosmos_RegisterConfiguration("COS_SIDEBARR", "CHECKBOX", 
			SIDEBAR_CHECK2, 
			SIDEBAR_CHECK2_INFO,
			SideBar_Toggle,
			Right_Sidebar_On
			);
		Cosmos_RegisterConfiguration("COS_SIDEBAR_HIDEKEYS", "CHECKBOX",
			SIDEBAR_HIDEKEYS,
			SIDEBAR_HIDEKEYS_INFO,
			SideBar_SetHideKeys,
			SideBar_HideKeys
			);
		Cosmos_RegisterConfiguration("COS_SIDEBAR_HIDEKEYMOD", "CHECKBOX",
			SIDEBAR_HIDEKEYMOD,
			SIDEBAR_HIDEKEYMOD_INFO,
			SideBar_SetHideKeyMod,
			SideBar_HideKeyMod
			);
		Cosmos_RegisterConfiguration(
			"COS_SIDEBAR_LEFTSLIDER", -- CVAr
			"SLIDER",
			SIDEBAR_CONFIG_SLIDER_LEFT,
			"",
			SideBar_LeftSliderUpdate,
			0,
			0,				--Default Value
			-500,			--Min value
			500,			--Max value
			SIDEBAR_CONFIG_SLIDER_OFFSET,		--Slider Text
			1,				--Slider Increment
			1,				--Slider state text on/off
			"",				--Slider state text append
			1				--Slider state text multiplier
			);
		Cosmos_RegisterConfiguration(
			"COS_SIDEBAR_RIGHTSLIDER", -- CVAr
			"SLIDER",
			SIDEBAR_CONFIG_SLIDER_RIGHT,
			"",
			SideBar_RightSliderUpdate,
			0,
			0,				--Default Value
			-500,			--Min value
			500,			--Max value
			SIDEBAR_CONFIG_SLIDER_OFFSET,		--Slider Text
			1,				--Slider Increment
			1,				--Slider state text on/off
			"",				--Slider state text append
			1				--Slider state text multiplier
			);
		Cosmos_RegisterConfiguration(
			"COS_SIDEBAR_ID_LEFTSLIDER", -- CVAr
			"SLIDER",
			SIDEBAR_CONFIG_SLIDER_ID_LEFT,
			"",
			SideBar_ID_LeftSliderUpdate,
			0,
			10,				--Default Value
			1,			--Min value
			10.5,			--Max value
			SIDEBAR_CONFIG_ID_SLIDER_OFFSET,		--Slider Text
			0.5,			--Slider Increment
			1,				--Slider state text on/off
			"",				--Slider state text append
			1				--Slider state text multiplier
			);
		Cosmos_RegisterConfiguration(
			"COS_SIDEBAR_ID_RIGHTSLIDER", -- CVAr
			"SLIDER",
			SIDEBAR_CONFIG_ID_SLIDER_RIGHT,
			"",
			SideBar_ID_RightSliderUpdate,
			0,
			10.5,				--Default Value
			1,			--Min value
			10.5,			--Max value
			SIDEBAR_CONFIG_ID_SLIDER_OFFSET,		--Slider Text
			0.5,			--Slider Increment
			1,				--Slider state text on/off
			"",				--Slider state text append
			1				--Slider state text multiplier
			);
		Cosmos_RegisterConfiguration(
			"COS_SIDEBAR_BUTTON_LEFTSLIDER", -- CVAr
			"SLIDER",
			SIDEBAR_CONFIG_SLIDER_BUTTON_LEFT,
			"",
			SideBar_LeftSliderNumberButtonsUpdate,
			1,
			SideBar2_NumberOfButtons,				--Default Value
			0,			--Min value
			12,			--Max value
			SIDEBAR_CONFIG_BUTTON_SLIDER_OFFSET,		--Slider Text
			1,			--Slider Increment
			1,				--Slider state text on/off
			"",				--Slider state text append
			1				--Slider state text multiplier
			);
		Cosmos_RegisterConfiguration(
			"COS_SIDEBAR_BUTTON_RIGHTSLIDER", -- CVAr
			"SLIDER",
			SIDEBAR_CONFIG_BUTTON_SLIDER_RIGHT,
			"",
			SideBar_RightSliderNumberButtonsUpdate,
			1,
			SideBar_NumberOfButtons,				--Default Value
			0,			--Min value
			12,			--Max value
			SIDEBAR_CONFIG_BUTTON_SLIDER_OFFSET,		--Slider Text
			1,			--Slider Increment
			1,				--Slider state text on/off
			"",				--Slider state text append
			1				--Slider state text multiplier
			);
	end
end
function SideBarButton_OnLoad()
	this.showgrid = 1;
	this.flashing = 0;
	this.flashtime = 0;
	SideBarButton_Update();
	this:RegisterForDrag("LeftButton", "RightButton");
	this:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	this:RegisterEvent("ACTIONBAR_SHOWGRID");
	this:RegisterEvent("ACTIONBAR_HIDEGRID");
	this:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
	this:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
	this:RegisterEvent("ACTIONBAR_UPDATE_STATE");
	this:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
	this:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	this:RegisterEvent("PLAYER_AURAS_CHANGED");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("UNIT_AURASTATE");
	this:RegisterEvent("UNIT_INVENTORY_CHANGED");
	this:RegisterEvent("CRAFT_SHOW");
	this:RegisterEvent("CRAFT_CLOSE");
	this:RegisterEvent("TRADE_SKILL_SHOW");
	this:RegisterEvent("TRADE_SKILL_CLOSE");
	this:RegisterEvent("UNIT_HEALTH");
	this:RegisterEvent("UNIT_MANA");
	this:RegisterEvent("UNIT_RAGE");
	this:RegisterEvent("UNIT_FOCUS");
	this:RegisterEvent("UNIT_ENERGY");
	this:RegisterEvent("PLAYER_ENTER_COMBAT");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("UNIT_COMBO_POINTS");
	this:RegisterEvent("UPDATE_BINDINGS");
	this:RegisterEvent("START_AUTOREPEAT_SPELL");
	this:RegisterEvent("STOP_AUTOREPEAT_SPELL");
	
	SideBarButton_UpdateHotkeys();

end

function SideBarButton_UpdateHotkeys(actionButtonType, button)
	if (not button) then
		button = this;
	end
	if ( not actionButtonType ) then
		actionButtonType = string.upper(button:GetParent():GetName()).."ACTIONBUTTON";
	end
	local hotkey = getglobal(button:GetName().."HotKey");
	local curID = button:GetID();
	if (SideBar_HideKeys==1) then
		hotkey:SetText("");
	else
		local action = actionButtonType..curID;	
		local keyText = KeyBindingFrame_GetLocalizedName(GetBindingKey(action), "KEY_");
		if (SideBar_HideKeyMod==1) then
			while ( ( keyText ) and ( strlen(keyText) > 1 ) and (string.find(keyText, "-")) and (keyText ~= "KP -") and (keyText ~= "Num Pad -")) do
				firsti, lasti, keyText = string.find (keyText, "-(.+)");
			end
		end
		if ( ( not keyText ) or ( strlen(keyText) <= 0 ) ) then keyText = ""; end
		hotkey:SetText(keyText);
	end
end

function SideBarButton_UpdateAllHotkeys()
	for i=1, SideBar_TotalNumberOfButtons do
		local button = getglobal("SideBarButton"..i);
		if (button) then
			SideBarButton_UpdateHotkeys(nil, button)
		end
	end
	for i=1, SideBar_TotalNumberOfButtons do
		local button = getglobal("SideBar2Button"..i);
		if (button) then
			SideBarButton_UpdateHotkeys(nil, button)
		end
	end
end

function SideBar_Update()
	for i=1, SideBar_TotalNumberOfButtons do
		local button = getglobal("SideBarButton"..i);
		if (button) then
			SideBarButton_UpdateButton(button);
		end
	end
end

function SideBar2_Update()
	for i=1, SideBar2_TotalNumberOfButtons do
		local button = getglobal("SideBar2Button"..i);
		if (button) then
			SideBarButton_UpdateButton(button);
		end
	end
end

function SideBarButton_ShouldBeVisibleByDefault(button)
	if ( not button ) then
		button = this;
	end
	local numberOfButtons = SideBar_NumberOfButtons;
	local name = button:GetName();
	if ( strfind(name, "SideBar2") ) then
		numberOfButtons = SideBar2_NumberOfButtons;
	end
	local id = button:GetID();
	if ( id > numberOfButtons ) then
		return false;
	else
		return true;
	end
end


function SideBarButton_UpdateButton(button)
	if ( not button ) then
		button = this;
	end
	local icon = getglobal(button:GetName().."Icon");
	local buttonCooldown = getglobal(button:GetName().."Cooldown");
	local texture = GetActionTexture(SideBarButton_GetID(button));
	if ( texture ) then
		icon:SetTexture(texture);
		icon:Show();
		button.rangeTimer = TOOLTIP_UPDATE_TIME;
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
		-- Save texture if the button is a bonus button, will be needed later
		--if ( button.isBonus ) then
		--	button.texture = texture;
		--end
	else
		icon:Hide();
		buttonCooldown:Hide();
		button.rangeTimer = nil;
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot");
	end

	SideBarButton_UpdateCount();
	if ( not SideBarButton_ShouldBeVisibleByDefault(button) ) then
		button:Hide();
	elseif ( HasAction(SideBarButton_GetID(button)) ) then
		button:Show();
		SideBarButton_UpdateUsable(button);
		SideBarButton_UpdateCooldown(button);
	elseif ( button.showgrid == 0 ) then
		button:Hide();
	else
		getglobal(button:GetName().."Cooldown"):Hide();
	end
	if ( IsAttackAction(SideBarButton_GetID(button)) and IN_ATTACK_MODE == 1 ) then
		SideBarButton_StartFlash(button);
	else
		SideBarButton_StopFlash(button);
	end
	if ( GameTooltip:IsOwned(button) ) then
		SideBarButton_SetTooltip(button);
	else
		button.updateTooltip = nil;
	end

	-- Update Macro Text
	local macroName = getglobal(button:GetName().."Name");
	macroName:SetText(GetActionText(SideBarButton_GetID(button)));
end

function SideBarButton_Update()
	SideBarButton_UpdateButton(this);
end

function SideBarButton_ShowGrid(button)
	if ( not button ) then
		button = this;
	end
	button.showgrid = button.showgrid+1;
	getglobal(button:GetName().."NormalTexture"):SetVertexColor(1.0, 1.0, 1.0);
	button:Show();
end

function SideBarButton_HideGrid(button)	
	if ( not button ) then
		button = this;
	end
	button.showgrid = button.showgrid-1;
	if ( button.showgrid == 0 and not HasAction(SideBarButton_GetID(button)) ) then
		button:Hide();
	end
end

function SideBarButton_UpdateState(button)
	if ( not button ) then
		button = this;
	end
	if ( IsCurrentAction(SideBarButton_GetID(button)) ) then
		button:SetChecked(1);
	else
		button:SetChecked(0);
	end
end

function SideBarButton_UpdateUsable(button)
	if ( not button ) then
		button = this;
	end
	local icon = getglobal(button:GetName().."Icon");
	local normalTexture = getglobal(button:GetName().."NormalTexture");
	local isUsable, notEnoughMana = IsUsableAction(SideBarButton_GetID(button));
	if ( isUsable ) then
		local inRange = true;
		if ( button.rangeTimer and (IsActionInRange(SideBarButton_GetID(button)) == 0)) then
			inRange = false;
		end
		if ( inRange ) then
			icon:SetVertexColor(1.0, 1.0, 1.0);
			normalTexture:SetVertexColor(1.0, 1.0, 1.0);
		else
			icon:SetVertexColor(1.0, 0.5, 0.5);
			normalTexture:SetVertexColor(1.0, 0.5, 0.5);
		end
	elseif ( notEnoughMana ) then
		icon:SetVertexColor(0.5, 0.5, 1.0);
		normalTexture:SetVertexColor(0.5, 0.5, 1.0);
	else
		icon:SetVertexColor(0.4, 0.4, 0.4);
		normalTexture:SetVertexColor(1.0, 1.0, 1.0);
	end
end

function SideBarButton_UpdateCount(button)
	if ( not button ) then
		button = this;
	end
	local text = getglobal(button:GetName().."Count");
	local count = GetActionCount(SideBarButton_GetID(button));
	if ( count > 1 ) then
		text:SetText(count);
	else
		text:SetText("");
	end
end

function SideBarButton_UpdateCooldown(button)
	if ( not button ) then
		button = this;
	end
	local cooldown = getglobal(button:GetName().."Cooldown");
	local start, duration, enable = GetActionCooldown(SideBarButton_GetID(button));
	CooldownFrame_SetTimer(cooldown, start, duration, enable);
end

function SideBarButton_OnEvent(event)
	if ( event == "ACTIONBAR_SLOT_CHANGED" ) then
		if ( arg1 == -1 or arg1 == SideBarButton_GetID(this) ) then
			SideBarButton_Update();
		end
		return;
	end
	if ( event == "PLAYER_AURAS_CHANGED") then
		SideBarButton_Update();
		SideBarButton_UpdateState();
		return;
	end
	if ( event == "ACTIONBAR_SHOWGRID" ) then
		SideBarButton_ShowGrid();
		return;
	end
	if ( event == "ACTIONBAR_HIDEGRID" ) then
		SideBarButton_HideGrid();
		return;
	end
	if ( event == "UPDATE_BINDINGS" ) then
		SideBarButton_UpdateHotkeys();
	end

	-- All event handlers below this line MUST only be valid when the button is visible
	if ( not this:IsVisible() ) then
		return;
	end

	if ( event == "PLAYER_TARGET_CHANGED" ) then
		SideBarButton_UpdateUsable();
		return;
	end
	if ( event == "UNIT_AURASTATE" ) then
		if ( arg1 == "player" or arg1 == "target" ) then
			SideBarButton_UpdateUsable();
		end
		return;
	end
	if ( event == "UNIT_INVENTORY_CHANGED" ) then
		if ( arg1 == "player" ) then
			SideBarButton_Update();
		end
		return;
	end
	if ( event == "ACTIONBAR_UPDATE_STATE" ) then
		SideBarButton_UpdateState();
		return;
	end
	if ( event == "ACTIONBAR_UPDATE_USABLE" ) then
		SideBarButton_UpdateUsable();
		return;
	end
	if ( event == "ACTIONBAR_UPDATE_COOLDOWN" ) then
		-- Do we really need to update usable state?  It adds 50% CPU usage in this event processing
		--ActionButton_UpdateUsable();
		SideBarButton_UpdateCooldown();
		return;
	end
	if ( event == "CRAFT_SHOW" or event == "CRAFT_CLOSE" or event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_CLOSE" ) then
		SideBarButton_UpdateState();
		return;
	end
	if ( arg1 == "player" and (event == "UNIT_HEALTH" or event == "UNIT_MANA" or event == "UNIT_RAGE" or event == "UNIT_FOCUS" or event == "UNIT_ENERGY") ) then
		SideBarButton_UpdateUsable();
		return;
	end
	if ( event == "PLAYER_ENTER_COMBAT" ) then
		IN_ATTACK_MODE = 1;
		if ( IsAttackAction(SideBarButton_GetID(this)) ) then
			SideBarButton_StartFlash();
		end
		return;
	end
	if ( event == "PLAYER_LEAVE_COMBAT" ) then
		IN_ATTACK_MODE = 0;
		if ( IsAttackAction(SideBarButton_GetID(this)) ) then
			SideBarButton_StopFlash();
		end
		return;
	end
	if ( (event == "UNIT_COMBO_POINTS") and (arg1 == "player") ) then
		SideBarButton_UpdateUsable();
		return;
	end
	if ( event == "START_AUTOREPEAT_SPELL" ) then
		IN_AUTOREPEAT_MODE = 1;
		if ( IsAutoRepeatAction(SideBarButton_GetID(this)) ) then
			SideBarButton_StartFlash();
		end
		return;
	end
	if ( event == "STOP_AUTOREPEAT_SPELL" ) then
		IN_AUTOREPEAT_MODE = nil;
		if ( SideBarButton_IsFlashing() and not IsAttackAction(SideBarButton_GetID(this)) ) then
			SideBarButton_StopFlash();
		end
		return;
	end
end

function SideBarButton_SetTooltip()
	if ( not button ) then
		button = this;
	end
	if ((GetCVar("UberTooltips") == "1") and (not Cosmos_RelocateUberTooltip)) then
		GameTooltip_SetDefaultAnchor(GameTooltip, button);
	else
		if (SmartSetOwner) then
			SmartSetOwner(button);
		else
			GameTooltip:SetOwner(button,"ANCHOR_TOPRIGHT");
		end
	end

	if ( GameTooltip:SetAction(SideBarButton_GetID(button)) ) then
		button.updateTooltip = TOOLTIP_UPDATE_TIME;
	else
		button.updateTooltip = nil;
	end
end

function SideBarButton_SetTooltipRight(button)
	if ( not button ) then
		button = this;
	end
	if ((GetCVar("UberTooltips") == "1") and (not Cosmos_RelocateUberTooltip)) then
		GameTooltip_SetDefaultAnchor(GameTooltip, button);
	else
		if (SmartSetOwner) then
			SmartSetOwner(button);
		else
			GameTooltip:SetOwner(button,"ANCHOR_TOPRIGHT");
		end
	end

	if ( GameTooltip:SetAction(SideBarButton_GetID(button)) ) then
		button.updateTooltip = TOOLTIP_UPDATE_TIME;
	else
		button.updateTooltip = nil;
	end
end

function SideBarButton_OnUpdate(elapsed, button)
	if ( not button ) then
		button = this;
	end
	if ( button.flashing ~= 0 and button.flashing) then
		button.flashtime = button.flashtime - elapsed;
		if ( button.flashtime <= 0 ) then
			local overtime = -button.flashtime;
			if ( overtime >= ATTACK_BUTTON_FLASH_TIME ) then
				overtime = 0;
			end
			button.flashtime = ATTACK_BUTTON_FLASH_TIME - overtime;

			local flashTexture = getglobal(button:GetName().."Flash");
			if ( flashTexture:IsVisible() ) then
				flashTexture:Hide();
			else
				flashTexture:Show();
			end
		end
	end
	
	-- Handle range indicator
	if ( button.rangeTimer ) then
		if ( button.rangeTimer < 0 ) then
			SideBarButton_UpdateUsable(button);
			local count = getglobal(button:GetName().."HotKey");
			if ( IsActionInRange( SideBarButton_GetID(button)) == 0 ) then
				if (count) then
					count:SetVertexColor(1.0, 0.1, 0.1);
				end
			else
				if (count) then
					count:SetVertexColor(0.6, 0.6, 0.6);
				end
			end
			button.rangeTimer = TOOLTIP_UPDATE_TIME;
		else
			button.rangeTimer = button.rangeTimer - elapsed;
		end
	else
		local count = getglobal(button:GetName().."HotKey");
		count:SetVertexColor(0.6, 0.6, 0.6);
	end


	if ( not button.updateTooltip ) then
		return;
	end

	button.updateTooltip = button.updateTooltip - elapsed;
	if ( button.updateTooltip > 0 ) then
		return;
	end

	if ( GameTooltip:IsOwned(button) ) then
		SideBarButton_SetTooltip(button);
	else
		button.updateTooltip = nil;
	end
end

function SideBarButton_GetID(button)
	if( button == nil ) then
		message("nil button passed into SideBarButton_GetID(), contact Jeff");
		return 0;
	end
	local name = button:GetName();
	local id = (button:GetID());
	if ( strfind(name, "SideBarButton") ) then
		id = id + SideBarButton_IDOffset;
	elseif ( strfind(name, "SideBar2Button") ) then
		id = id + SideBar2Button_IDOffset;
	end
	return id;
end

function SideBarButton_StartFlash(button)
	if ( not button ) then
		button = this;
	end
	button.flashing = 1;
	button.flashtime = 0;
	SideBarButton_UpdateState();
end

function SideBarButton_StopFlash(button)
	if ( not button ) then
		button = this;
	end
	button.flashing = 0;
	getglobal(button:GetName().."Flash"):Hide();
	SideBarButton_UpdateState();
end


function SideBarButton_IsFlashing(button)
	if ( not button ) then
		button = this;
	end
	if ( button.flashing == 1 ) then
		return 1;
	else
		return nil;
	end
end

function SideBar_Toggle ( toggle ) if (toggle == 1) then SideBar:Show() else SideBar:Hide(); end end
function SideBar2_Toggle ( toggle ) if (toggle == 1) then SideBar2:Show() else SideBar2:Hide(); end end

function SideBar_SetHideKeys(checked)
	if (checked and (checked==1)) then
		if (SideBar_HideKeys ~= 1) then
			SideBar_HideKeys=1;
			SideBarButton_UpdateAllHotkeys();
		end
	else
		if (SideBar_HideKeys ~= 0) then
			SideBar_HideKeys=0;
			SideBarButton_UpdateAllHotkeys();
		end
	end
end

function SideBar_SetHideKeyMod(checked)
	if (checked and (checked==1)) then
		if (SideBar_HideKeyMod ~= 1) then
			SideBar_HideKeyMod=1;
			SideBarButton_UpdateAllHotkeys();
		end
	else
		if (SideBar_HideKeyMod ~= 0) then
			SideBar_HideKeyMod=0;
			SideBarButton_UpdateAllHotkeys();
		end
	end
end